@@ -1,5 +1,5 @@
 ;; herd.scm -- The program to herd the Shepherd.
-;; Copyright (C) 2013, 2014, 2016, 2018 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2016, 2018, 2019 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Jährling <wolfgang@pro-linux.de>
 ;;
 ;; This file is part of the GNU Shepherd.
@@ -83,6 +83,10 @@ of pairs."
        (if running
            (begin
              (format #t (l10n "  It is started.~%"))
+
+             ;; TRANSLATORS: The "~s" bit is most of the time a placeholder
+             ;; for the PID (an integer) of the running process, and
+             ;; occasionally for another Scheme object.
              (format #t (l10n "  Running value is ~s.~%") running))
            (format #t (l10n "  It is stopped.~%")))
        (if enabled?
@@ -150,8 +154,10 @@ the daemon via SOCKET-FILE."
                            service)
              (exit 1)))
           (_
-           ;; For other commands, we don't do any interpretation.
-           #t)))
+           ;; For other commands, exit successfully if and only if all the
+           ;; values of RESULT are true.
+           (unless (every ->bool result)
+             (exit 1)))))
        (('reply ('version 0 _ ...)                ;an error
                 ('result _) ('error error)
                 ('messages messages))
