@@ -1,5 +1,5 @@
 # GNU Shepherd --- Test basic communication capabilities.
-# Copyright © 2013, 2014, 2016, 2017, 2018 Ludovic Courtès <ludo@gnu.org>
+# Copyright © 2013, 2014, 2016, 2017, 2018, 2019 Ludovic Courtès <ludo@gnu.org>
 # Copyright © 2016 Mathieu Lirzin <mthl@gnu.org>
 # Copyright © 2014 Alex Sassmannshausen <alex.sassmannshausen@gmail.com>
 #
@@ -57,7 +57,8 @@ cat > "$conf"<<EOF
    #:actions (make-actions (hi "Say hi."
                                (lambda _
                                  (display "start\n\nend\n")
-                                 #t)))
+                                 #t))
+			   (fail "Fail." (const #f)))
    #:respawn? #f)
  (make <service>
    #:provides '(broken)
@@ -90,6 +91,10 @@ $herd stop test
 
 $herd status test | grep stopped
 
+# Stopping a stopped service should be a no-op.
+$herd stop test
+test -z "`$herd stop test 2>&1`"
+
 # Disable a service and make sure it cannot be started.
 $herd disable test-2
 if $herd start test-2
@@ -111,6 +116,9 @@ $herd hi test-2 | grep '^start$'
 $herd hi test-2 | grep '^end$'
 test `$herd hi test-2 | wc -l` -eq 3
 
+# An action that returns false must lead to a non-zero exit code.
+if $herd fail test-2; then false; else true; fi
+
 # This used to crash shepherd: <http://bugs.gnu.org/24684>.
 if $herd enable test-2 with extra arguments
 then false; else true; fi
